unit uAnsiStrings;
{$DEFINE ASSEMBLER}
{$IFDEF FPC}
  {$MODE DELPHI}
{$ENDIF}

interface

function LowCase(ch: Char): Char;
function StrScan(const Str: PChar; Chr: Char): PChar;
function StrLen(const Str: PChar): Cardinal;
function StrEnd(const Str: PChar): PChar;
function StrCat(Dest: PChar; const Source: PChar): PChar;
function StrLCat(Dest: PChar; const Source: PChar; MaxLen: Cardinal): PChar;
function StrCopy(Dest: PChar; const Source: PChar): PChar;
function StrECopy(Dest: PChar; const Source: PChar): PChar;
function StrLCopy(Dest: PChar; const Source: PChar; MaxLen: Cardinal): PChar; 
function StrAlloc(Size: Cardinal): PChar;
procedure StrDispose(Str: PChar);

implementation

{$IFDEF FPC}
  {$UNDEF ASSEMBLER}
{$ENDIF}

{$IFDEF ASSEMBLER}
function LowCase(ch: Char): Char; assembler;
asm
{ ->    AL      Character       }
{ <-    AL      Result          }

        CMP     AL,'A'
        JB      @@exit
        CMP     AL,'Z'
        JA      @@exit
        ADD     AL,'a' - 'A'
@@exit:
end;
{$ELSE}
function LowCase(ch: Char): Char;
begin
  if (ch >= 'A') and (ch <= 'Z') then
    Result := Char(Byte(ch) + Byte('a') - Byte('A'))
  else
    Result := ch;
end;
{$ENDIF}

{$IFDEF ASSEMBLER}
function StrScan(const Str: PChar; Chr: Char): PChar; assembler;
asm
        PUSH    EDI
        PUSH    EAX
        MOV     EDI,Str
        MOV     ECX,0FFFFFFFFH
        XOR     AL,AL
        REPNE   SCASB
        NOT     ECX
        POP     EDI
        MOV     AL,Chr
        REPNE   SCASB
        MOV     EAX,0
        JNE     @@1
        MOV     EAX,EDI
        DEC     EAX
@@1:    POP     EDI
end;
{$ELSE}
function StrScan(const Str: PChar; Chr: Char): PChar;
begin
  Result := Str;
  while (Result^ <> #0) and (Result^ <> Chr) do
    Inc(Result);
  if (Result^ = #0) then
    Result := nil;
end;
{$ENDIF}

{$IFDEF ASSEMBLER}
function StrLen(const Str: PChar): Cardinal; assembler;
asm
        MOV     EDX,EDI
        MOV     EDI,EAX
        MOV     ECX,0FFFFFFFFH
        XOR     AL,AL
        REPNE   SCASB
        MOV     EAX,0FFFFFFFEH
        SUB     EAX,ECX
        MOV     EDI,EDX
end;
{$ELSE}
function StrLen(const Str: PChar): Cardinal;
var P: PChar;
begin
  Result := 0;
  P := Str;
  while P^ <> #0 do
    begin
    Inc(p);
    Inc(Result);
    end;
end;
{$ENDIF}

{$IFDEF ASSEMBLER}
function StrEnd(const Str: PChar): PChar; assembler;
asm
        MOV     EDX,EDI
        MOV     EDI,EAX
        MOV     ECX,0FFFFFFFFH
        XOR     AL,AL
        REPNE   SCASB
        LEA     EAX,[EDI-1]
        MOV     EDI,EDX
end;
{$ELSE}
function StrEnd(const Str: PChar): PChar;
begin
  Result := Str;
  while Result^ <> #0 do
    Inc(Result);
end;
{$ENDIF}

function StrCat(Dest: PChar; const Source: PChar): PChar;
begin
  StrCopy(StrEnd(Dest), Source);
  Result := Dest;
end;

{$IFDEF ASSEMBLER}
function StrLCat(Dest: PChar; const Source: PChar; MaxLen: Cardinal): PChar; assembler;
asm
        PUSH    EDI
        PUSH    ESI
        PUSH    EBX
        MOV     EDI,Dest
        MOV     ESI,Source
        MOV     EBX,MaxLen
        CALL    StrEnd
        MOV     ECX,EDI
        ADD     ECX,EBX
        SUB     ECX,EAX
        JBE     @@1
        MOV     EDX,ESI
        CALL    StrLCopy
@@1:    MOV     EAX,EDI
        POP     EBX
        POP     ESI
        POP     EDI
end;
{$ELSE}
function StrLCat(Dest: PChar; const Source: PChar; MaxLen: Cardinal): PChar;
begin
  MaxLen := MaxLen - StrLen(Source);
  Result := StrEnd(Dest);
  if MaxLen > 0 then
    StrLCopy(Result, Source, MaxLen);
  Result := Dest;
end;
{$ENDIF}

{$IFDEF ASSEMBLER}
function StrLCopy(Dest: PChar; const Source: PChar; MaxLen: Cardinal): PChar; assembler;
asm
        PUSH    EDI
        PUSH    ESI
        PUSH    EBX
        MOV     ESI,EAX
        MOV     EDI,EDX
        MOV     EBX,ECX
        XOR     AL,AL
        TEST    ECX,ECX
        JZ      @@1
        REPNE   SCASB
        JNE     @@1
        INC     ECX
@@1:    SUB     EBX,ECX
        MOV     EDI,ESI
        MOV     ESI,EDX
        MOV     EDX,EDI
        MOV     ECX,EBX
        SHR     ECX,2
        REP     MOVSD
        MOV     ECX,EBX
        AND     ECX,3
        REP     MOVSB
        STOSB
        MOV     EAX,EDX
        POP     EBX
        POP     ESI
        POP     EDI
end;
{$ELSE}
function StrLCopy(Dest: PChar; const Source: PChar; MaxLen: Cardinal): PChar;
var Src: PChar;
begin
  Result := Dest;
  Src := Source;
  while (MaxLen > 0) and (Src^ <> #0) do
    begin
    Result^ := Src^;
    Inc(Result);
    Inc(Src);
    Dec(MaxLen);
    end;
  Result := Dest;
end;
{$ENDIF}

{$IFDEF ASSEMBLER}
function StrCopy(Dest: PChar; const Source: PChar): PChar; assembler;
asm
        PUSH    EDI
        PUSH    ESI
        MOV     ESI,EAX
        MOV     EDI,EDX
        MOV     ECX,0FFFFFFFFH
        XOR     AL,AL
        REPNE   SCASB
        NOT     ECX
        MOV     EDI,ESI
        MOV     ESI,EDX
        MOV     EDX,ECX
        MOV     EAX,EDI
        SHR     ECX,2
        REP     MOVSD
        MOV     ECX,EDX
        AND     ECX,3
        REP     MOVSB
        POP     ESI
        POP     EDI
end;
{$ELSE}
function StrCopy(Dest: PChar; const Source: PChar): PChar;
var Src: PChar;
begin
  Result := Dest;
  Src := Source;
  while (Src^ <> #0) do
    begin
    Result^ := Src^;
    Inc(Result);
    Inc(Src);
    end;
  Result := Dest;
end;
{$ENDIF}

{$IFDEF ASSEMBLER}
function StrECopy(Dest: PChar; const Source: PChar): PChar; assembler;
asm
        PUSH    EDI
        PUSH    ESI
        MOV     ESI,EAX
        MOV     EDI,EDX
        MOV     ECX,0FFFFFFFFH
        XOR     AL,AL
        REPNE   SCASB
        NOT     ECX
        MOV     EDI,ESI
        MOV     ESI,EDX
        MOV     EDX,ECX
        SHR     ECX,2
        REP     MOVSD
        MOV     ECX,EDX
        AND     ECX,3
        REP     MOVSB
        LEA     EAX,[EDI-1]
        POP     ESI
        POP     EDI
end;
{$ELSE}
function StrECopy(Dest: PChar; const Source: PChar): PChar;
begin
  Result := StrEnd(StrCopy(Dest, Source));
end;
{$ENDIF}

function StrAlloc(Size: Cardinal): PChar;
begin
  Inc(Size, SizeOf(Cardinal));
  GetMem(Result, Size);
  Cardinal(Pointer(Result)^) := Size;
  Inc(Result, SizeOf(Cardinal));
end;

procedure StrDispose(Str: PChar);
begin
  if Str <> nil then
  begin
    Dec(Str, SizeOf(Cardinal));
    FreeMem(Str, Cardinal(Pointer(Str)^));
  end;
end;

end.

